#!/usr/bin/env python3
#
# make_movie.py creates MPEG4 movies for cytosim
#
# Copyright F. J. Nedelec, 2007 - 2023
#
# To make MP4 Quicktime movies, you need to install ffmpeg:
# http://www.ffmpeg.org
# via Macports: sudo port install ffmpeg
# via Brew: brew install ffmpeg
#
# Note that given a set of images, one can assemble them directly with ffmpeg
# Movie with a subjective quality (lower is better), use -crf X, with X in [0, 51]:
# ffmpeg -r 12 -i image%04d.png -pix_fmt yuv420p -c:v libx264 -crf 23 movie.mp4
#
# Movie with fixed bit rate:
# ffmpeg -r 12 -i image%04d.png -pix_fmt yuv420p -c:v libx264 -b:v 2000k movie.mp4
#

"""
    Create movies for cytosim in given directories

Syntax:

    make_movie.py executable-with-arguments [options] [directories]

Procedure:
    
    In each directory, this will:
      1- call the executable to generate images
      2- invoke ffmpeg or other tools to assemble a movie
      3- delete images created in step 1
    
    If images of the right format are already present in the directory,
    you may specify an empty executable and step 1 and 3 will be skipped.

    The current directory is used if none is specified.
    
Options are specified as 'option=value', without space around the '=' sign.
Existing options and their values:

    codec     mpeg4, h264, h265   movie codec (default = 'h264')
    rate      integer             images per second (default = 12)
    quality   integer (default=3) subjective quality for MPEG4: 1=great, 4=good
                                  data rate for Quicktime (eg. 64)
    
    cleanup   0 or 1 (default=1)  remove temporary files (default = 1)
    lazy      0 or 1 (default=1)  bail out if output file already exists

Examples:

    make_movie.py 'play window_size=512,256' run*
    make_movie.py '~/bin/play3 zoom=2' run*
    make_movie.py imgs codec=265

History:
    Created by F. Nedelec, 14.12.2007
    Improved by Beat Rupp, March 2010
    Revised on March 19 2011 and Sept-Nov 2012 by F. Nedelec.
    F. Nedelec, 10.2013: images are created in sub-directories
    F. Nedelec, 12.2013, 9.2014, 04.2016, 15.03.2020, 10.01.2022
    
    https://ffmpeg.org/ffmpeg.html
"""

try:
    import sys, os, shutil, subprocess
except ImportError:
    print("  Error: could not load necessary python modules\n")
    sys.exit()

def executable(arg):
    return os.path.isfile(arg) and os.access(arg, os.X_OK)

# some parameters:
tool    = []
codec   = 'h264'
rate    = 12
lazy    = 1
cleanup = True
quality = '3'
tmp_dir = 'tmp'

prefix = 'make_movie.py:  '
err = sys.stderr

#-------------------------------------------------------------------------------

def copyFiles(files, dir):
    """
        rename files to 'image????.EXT' where '????' are consecutive numbers
    """
    cnt = 0
    res = []
    for f in files:
        [main, ext] = os.path.splitext(f)
        name = dir + '/image%04i' % cnt + ext
        if not f == name:
            shutil.copyfile(f, name)
            #err.write("    %s --> %s\n" % ( f, name ))
        res.append(name)
        cnt += 1
    return res

#-------------------------------------------------------------------------------

def makeImages(format):
    """
        call `tool` to generate images in temporary directory
    """
    val = subprocess.call(tool + ['movie', 'image_dir='+tmp_dir, 'image_format='+format], stderr=None)
    if val:
        raise IOError("`%s' failed with value %i\n" % (tool[0], val))


def makeImagesUnzip(format, src='objects.cmo'):
    """
        unzip cytosim's input if necessary to generate the images
    """
    tmp_file = ''
    if not os.path.isfile(src):
        tmp_file = src + '.gz'
        if os.path.isfile(tmp_file):
            cmd = "gunzip %s -c > %s" % (tmp_file, src)
            subprocess.call(cmd, shell=True)
        else:
            tmp_file = ''
    if not os.path.isfile(src):
        raise IOError("file '%s' not found!\n" % src)
    makeImages(format)
    if tmp_file:
        os.remove(tmp_file)


def getImages(path, format):
    """
        Get images or call makeImage() to generate them
    """
    if tool and os.access(tool[0], os.X_OK):
        makeImagesUnzip(format)
        images = [os.path.join(tmp_dir, s) for s in os.listdir(tmp_dir)]
        images = sorted(images)
    else:
        # search for images already made:
        import glob
        images = sorted(glob.glob('m*[0-9].'+format))
        if not images:
            images = sorted(glob.glob('i*[0-9].'+format))
        err.write(prefix+"using %i images found inside `%s'\n" % (len(images), path))
    if not images:
        raise IOError("could not produce images!")
    return copyFiles(images, tmp_dir)


def getImageSize(file):
    """
        Call ffprobe, and parse output to extract size of video
    """
    res = [256, 256];
    proc = subprocess.Popen(['ffprobe', '-v', 'quiet', '-show_streams', file], stdout=subprocess.PIPE)
    if not proc.wait():
        for line in proc.stdout:
            [key, equal, value] = line.partition('=')
            if key=="width":
                res[0] = int(value)
            elif key=="height":
                res[1] = int(value)
    return res

#-------------------------------------------------------------------------------

def makeMovie(path, filename):
    """
        create movie.mp4 from PNG or PPM files in the current directory
        This entirely relies on ffmpeg
    """
    pattern = tmp_dir+'/image%04d.png'
    images = getImages(path, 'png')
    if not images:
        #print('looking for PPM images')
        pattern = tmp_dir+'/image%04d.ppm'
        images = getImages(path, 'ppm')
    if not images:
        raise IOError("no suitable images found")
    # build arguments for 'ffmpeg':
    args = ['ffmpeg', '-v', 'quiet', '-r', '%i'%rate, '-i', pattern]
    if codec == 'h265':
        args.extend(['-vcodec', 'libx265', '-tag:v', 'hvc1'])
    elif codec == 'h264':
        args.extend(['-vcodec', 'libx264'])
    else:
        args.extend(['-vcodec', 'mpeg4'])
    args.extend(['-pix_fmt', 'yuv420p', '-q:v', quality, filename])
    print(prefix + ' '.join(args))
    val = subprocess.call(args) #, stdout=None,stderr=None)
    if val:
        raise IOError("`ffmpeg` failed with value %i\n  %s\n" % (val, ' '.join(args)))

#-------------------------------------------------------------------------------

def process(path, filenames):
    """
        assemble movie in given directory (`path`)
    """
    global tmp_dir, cleanup
    res = 'movie.mp4'
    if ( res in filenames ) and lazy:
        err.write(prefix+"bailing out: `%s/%s' already exists!\n" % (path, res))
        return ''
    err.write("\n")
    try:
        import tempfile
        tmp_dir = tempfile.mkdtemp('', 'imgs-', '.')
        #err.write(prefix+"created directory %s\n" % tmp_dir)
    except Exception as e:
        err.write(prefix+"could not make temporary directory %s\n" % repr(e))
    try:
        makeMovie(path, res)
    except Exception as e:
        err.write(prefix+" %s\n" % str(e))
        return ''
    if cleanup:
        shutil.rmtree(tmp_dir)
        #err.write(prefix+"deleted directory %s\n" % tmp_dir)
    else:
        err.write(prefix+"folder `%s' contains generated images\n" % tmp_dir)
    return res


def process_dir(path):
    """call process() with appropriate arguments"""
    files = []
    with os.scandir() as it:
        for e in it:
            if e.is_file():
                files.append(e.name)
    return process(path, files)
    

#-------------------------------------------------------------------------------

def main(args):
    """
        process command line arguments
    """
    global tool, cleanup, lazy, codec, rate, quality
    paths = []
    
    try:
        arg = args[0]
    except:
        err.write(prefix+"you must specify an executable or a directory containing images\n")
        sys.exit()

    arg0 = os.path.expanduser(arg.split()[0])
    if os.access(arg0, os.X_OK) and os.path.isfile(arg0):
        tool = arg.split()
        tool[0] = os.path.abspath(arg0)
    elif os.path.isdir(arg):
        paths.append(arg)
    else:
        err.write(prefix+"You must specify an executable or a directory containing images\n")
        sys.exit()

    for arg in args[1:]:
        [key, equal, value] = arg.partition('=')
        
        if key=='' or equal!='=' or value=='':
            if os.path.isdir(arg):
                paths.append(arg)
            elif arg=='+':
                codec = 'h264'
            elif arg=='++':
                codec = 'h265'
            else:
                err.write(prefix+"ignored '%s' on command line\n" % arg)
        else:
            if key=='rate':
                rate = int(value)
            elif key=='cleanup':
                cleanup = bool(value)
            elif key=='codec':
                codec = value
            elif key=='quality':
                quality = value
            elif key=='lazy':
                lazy = int(value)
            else:
                err.write(prefix+"ignored '%s' on command line\n" % arg)

    if not paths:
        paths.append('.')

    cwd = os.getcwd()
    res = ''
    for p in paths:
        os.chdir(p)
        res = process_dir(p)
        os.chdir(cwd)
        # move file to parent directory if only one path was specified
        if res and len(paths) == 1 and not os.path.isfile(res):
            os.rename(os.path.join(paths[0],res), os.path.basename(res))
            err.write(prefix+"created %s\n" % res)
        else:
            err.write(prefix+"created %s/%s\n" % (p, res))

#-------------------------------------------------------------------------------

if __name__ == "__main__":
    if len(sys.argv) > 1 and sys.argv[1].endswith("help"):
        print(__doc__)
    else:
        main(sys.argv[1:])




#!/usr/bin/env python3
#
# read_config.py formats Cytosim's configuration files
#
# FJN 13-15.11.2013 - 31.07.2014, 3.09.2015, 12.08.2018, 11.12.2019, 27.05.2021

"""
    Read cytosim configuration files

Syntax:
   
    read_config.py FILE
    read_config.py - FILE
    
Description:

    Reads a config file and prints a formatted copy to standard output.
    Two formats are possible: vertical (default) or horizontal (second syntax)

F. Nedelec 11.2013 --- 27.05.2021
"""

import sys, os, io

read_comments = False
column_width = 0
err = sys.stderr
out = sys.stdout


def format_values(val):
    """
    Convert values to a cannonical ASCII representation
    """
    if isinstance(val, str):
        return val
    if isinstance(val, dict):
        r = "( "
        for k in sorted(val):
            r += str(k) + '=' + format_values(val[k]) + "; "
        return r + ")"
    try:
        r = format_values(val[0])
        e = len(val) - 1
        if isinstance(val[e], str) and val[e][0] == '%':
            e = e - 1
        for k in val[1:e+1]:
            r += ", " + format_values(k)
        return r
    except:
        return repr(val)


class Instruction:
    words_ = []
    values_ = {}
    code_ = {}
    
    def __init__(self, key):
        self.words_ = [ key ]
    
    def value(self, key, index=0):
        if key in self.vals:
            val = self.vals[key]
            if isinstance(val, list):
                return val[index]
            else:
                return val
        return ""
    
    def values(self):
        return self.values_;
    
    def __repr__(self):
        return self.format_horizontal()
    
    def format_header(self):
        res = ''
        for w in self.words_:
            res += w + ' '
        return res
    
    def format_horizontal(self):
        res = self.format_header() + "{ "
        if self.code_:
            for i in self.code_:
                res += i.format_horizontal() + "; "
        else:
            for p in sorted(self.values_):
                res += str(p) + '=' + format_values(self.values_[p]) + "; "
        res += "}"
        return res
    
    def format_vertical(self, indent = ""):
        res = indent + self.format_header() + "\n" + indent + "{\n"
        if self.code_:
            for i in self.code_:
                res += i.format_vertical(indent+"   ")
        else:
            for p in sorted(self.values_):
                lin = indent + "   " + p.ljust(column_width) + " = " + format_values(self.values_[p])
                #add line-comment if present:
                try:
                    e = self.values_[p][-1]
                    if e[0] == '%':
                        lin = (lin+';').ljust(50) + e
                    else:
                        lin += ';'
                except:
                    lin += ';'
                res += lin + '\n'
        res += indent + "}\n"
        return res
    
    def format(self, mode):
        if mode == 1:
            return self.format_horizontal()
        else:
            return self.format_vertical()



def get_char(fid):
    return fid.read(1)


def get_hexadecimal(fid, s):
    res = s
    pos = fid.tell()
    c = get_char(fid)
    while c and c in "0123456789ABCDEFabcdef":
        res += c
        pos = fid.tell()
        c = get_char(fid)
    fid.seek(pos, 0)
    return res


def get_number(fid, s):
    """
    Read a number with decimal point and optional exponent
    """
    res = s
    pos = fid.tell()
    c = get_char(fid)
    if c == 'x':
        return get_hexadecimal(fid, s+c)
    if c == '-' or c == '+':
        res += c
        pos = fid.tell()
        c = get_char(fid)
    no_point = 1
    while c.isdigit() or ( c=='.' and no_point ):
        if c=='.':
            no_point = 0
        res += c
        pos = fid.tell()
        c = get_char(fid)
    if c == 'e' or c == 'E':
        return res + get_number(fid, 'e')
    if c == 'P':
        return res
    fid.seek(pos, 0)
    return res


def delimiter(c):
    if c == '(': return ')'
    if c == '{': return '}'
    if c == '[': return ']'
    if c == '"': return '"'
    return 0


def get_until(fid, e):
    res = ''
    c = get_char(fid)
    while c and c != e:
        res += c
        c = get_char(fid)
    return res


def get_block(fid, s, e):
    """
    Return a block including the enclosing delimiters
    """
    res = s
    c = get_char(fid)
    while c:
        if c == e:
            res += c
            return res
        if delimiter(c):
            res += get_block(fid, c, delimiter(c))
        else:
            res += c
        c = get_char(fid)
    err.write("  Error: unmatched delimiter "+s+"\n")
    sys.exit(1)
    return res


def valid_token_char(c):
    return c.isalnum() or c == ':' or c == '_' or c == '.'


def get_token(fid):
    """
    Extract the next token from the file
    """
    c = get_char(fid)
    while c.isspace() and c != '\n':
        c = get_char(fid)
    if c == '%':
        return '%' + fid.readline()
    if delimiter(c):
        return get_block(fid, c, delimiter(c))
    if c.isdigit() or c == '-' or c == '+':
        return get_number(fid, c)
    res = c
    if valid_token_char(c):
        while c:
            pos = fid.tell()
            c = get_char(fid)
            if not valid_token_char(c):
                fid.seek(pos, 0)
                break
            res += c
    #print(" TOKEN |"+res+"|")
    return res


def uncode(arg):
    try:
        if isinstance(arg, unicode):
            return str(arg.decode('utf-8'))
    except:
        pass
    return arg


def convert(arg):
    """
    Convert a string to a int or float if possible
    """
    try:
        return int(arg)
    except:
        pass
    try:
        return float(arg)
    except:
        pass
    #remove spaces:
    s = arg.strip()
    #remove outer delimiters:
    if len(s) > 3 and delimiter(s[0]) == s[-1]:
        s = s[1:-1].strip()
    return s


def simplify(arg):
    """
    Simplify the values of imbricated sets and dictionaries
    """
    #print('SIMPLIFY   ', type(arg), arg)
    arg = uncode(arg)
    if isinstance(arg, str):
        return convert(arg)
    if isinstance(arg, dict):
        s = {}
        for k in arg:
            uk = uncode(k)
            s[uk] = simplify(arg[k])
        #print('SIMPLIFY=> ', s)
        return s
    try:
        if len(arg) == 1:
            return simplify(arg[0])
        s = []
        for a in arg:
            s.append(simplify(a))
        #print('SIMPLIFY=> ', s)
        return s
    except:
        return arg


def file_object(arg):
    #print("new_file_object", type(arg))
    try:
        return io.StringIO(arg)
    except:
        return io.StringIO(unicode(arg))


def read_list(fid):
    """
    Process a list of key=values
    """
    dic = {}
    key = ''
    val = ''
    kkk = ''
    tok = get_token(fid)
    while tok:
        #print('key %10s token |%s|' %(key, tok))
        if tok == '=':
            key = kkk
            dic[key] = []
            val = ''
        elif tok == '\n' or tok == ';' or tok[0] == '%':
            if key:
                dic[key].append(val)
                val = ''
                key = ''
            if tok[0] == '%':
                global read_comments
                if read_comments and kkk and len(tok) > 1:
                    dic[kkk].append(tok)
            if not tok == ';':
                kkk = ''
        elif tok == ',':
            if key:
                dic[key].append(val)
                val = ''
        elif delimiter(tok[0]):
            val = read_list(file_object(tok[1:-1]))
            if not val:
                val = tok
        else:
            if key:
                if val:
                    val += ' ' + tok
                else:
                    val = tok
            else:
                kkk = tok
        tok = get_token(fid)
    if key and val:
        dic[key].append(val)
    return dic


def parse_config(fid):
    """
    return a list resulting from parsing the specified file
    """
    keywords = [ 'set', 'new', 'run', 'call', 'delete', 'change', 'cut', 'mark', 'report', 'write', 'repeat' ];
    cur = []
    pile = []
    while fid:
        tok = get_token(fid)
        #print(">>>>> |"+tok+"|")
        if not tok:
            break
        if tok[0] == '%':
            pass
        elif tok == '\n':
            pass
        elif tok in keywords:
            if cur:
                pile.append(cur)
            cur = Instruction(tok)
        elif tok[0] == '{' or tok[0] == '(':
            if cur.words_[0] == 'repeat':
                sfid = io.StringIO(tok[1:-1])
                cur.code_ = parse_config(sfid)
            else:
                pam = read_list(file_object(tok[1:-1]))
                #print("VALUES  : ", pam)
                cur.values_ = simplify(pam)
                #print("SIMPLIFIED: ", cur.values_)
            pile.append(cur)
            cur = []
        elif cur and ( tok[0].isalpha() or tok.isdigit() or tok=='*' ):
            cur.words_.append(tok)
        else:
            print(">>>>>ignored |"+tok+"|")
    return pile


def parse(arg):
    """
    Returns a dictionnary obtained by parsing the file specified by name `arg`
    """
    if isinstance(arg, str):
        with open(arg, 'r') as f:
            pile = parse_config(f)
            f.close()
    else:
        pile = parse_config(arg)
    return pile

#------------------------------------------------------------------------

def format_config(pile, mode=0, prefix=''):
    """
    Returns a compact representation of the hierachical dictionnary
    """
    res = ''
    for i in pile:
        res += prefix + i.format(mode) + "\n"
    return res


def matching(ins, keys):
    """
    """
    j = 0
    res = 1
    for i in range(3):
        w = ins.words_[j]
        if w.isdigit():
            j += 1
            w = ins.words_[j]
        res &= ( keys[i]=='*' or w==keys[i] )
    return res


def get_command(pile, keys):
    """
    Return all commands corresponding to = [ command, class, name ]
    The keys can be '*' to specify a wildcard matching
    """
    if len(keys) != 3:
        return "Error: get_command(arg, keys) expects `keys` to be of size 3"
    res = []
    for p in pile:
        if matching(p, keys):
            res.append(p)
    if len(res) == 1:
        return res[0]
    return res


def get_value(arg, keys):
    """
    Return the value speficied by keys = [ command, class, name, parameter ]
    The first 3 keys can be '*' for wildcard matching
    """
    if len(keys) != 4:
        return "Error: get_value(arg, keys) expects `keys` to be of size 4"
    if isinstance(arg, str):
        pile = parse(arg)
    else:
        pile = arg
    for p in pile:
        if matching(p, keys):
            try:
                return p.values_[keys[3]]
            except KeyError:
                if keys[3]=='*':
                    return p.values_
    return "Unspecified"


#------------------------------------------------------------------------

def main(args):
    global read_comments;
    horizontal = False
    files = []
    
    for arg in args:
        if os.path.isfile(arg):
            files.append(arg)
        elif arg=="-" or arg=="-h":
            horizontal = True
        elif arg=="+":
            read_comments = True
        else:
            err.write("  Error: unexpected argument `%s'\n" % arg)
            sys.exit()
    
    if not files:
        err.write("  Error: you must specify a file to read\n")
        sys.exit()

    for f in files:
        out.write("\n")
        pile = parse(f)
        out.write(format_config(pile, horizontal))
        #print(f.rjust(40) + " " + value(pile, ['set', 'space', '*', 'geometry']) )


if __name__ == "__main__":
    if len(sys.argv) < 2 or sys.argv[1].endswith("help"):
        print(__doc__)
    else:
        main(sys.argv[1:])



#!/usr/bin/env python
#
# cleanup.py
#
# copyright F. Nedelec, December 14th 2007

"""
    Scan recursively from current directory,
    and clean up temporary cytosim files
    
Syntax:
    
    cleanup.py [walk=1] [commit=1] directory-list
    
Example:

    cleanup.py data*

F. Nedelec, April 2013
"""

import sys, os, shutil, subprocess

commit = 0

#------------------------------------------------------------------------

def process(path, dirs, files):
    """clean a directory"""
    print('- '*32+path)
    if not files:
        if not dirs:
            print('removed empty directory '+path)
            os.rmdir(path)
            return
        print('no files in directory '+path)
        return
    cdir = os.getcwd()
    os.chdir(path)
    if commit:
        for name in files:
            #print("FILE="+name)
            if name.endswith('~'):
                os.remove(name)
            elif name.endswith('.pyc'):
                os.remove(name)
            elif name == 'array.pbs':
                os.remove(name)
            elif name == "log.txt":
                os.remove(name)
            elif name.endswith('.o') or name.endswith('.a'):
                os.remove(name)
            elif name.startswith('makefile') and name.endswith('.dep'):
                os.remove(name)
            elif name.startswith('._'):
                try:
                    os.remove(name)
                except OSError:
                    pass
            elif name == '.DS_Store':
                os.remove(name)
            elif name == 'config.out' and os.path.exists('config.cym'):
                os.remove(name)
            else:
                try:
                    if 0 == os.path.getsize(name):
                        os.remove(name)
                except OSError:
                    pass
    os.chdir(cdir)


def process_dir(path):
    """call process() with appropriate arguments"""
    files = os.listdir(path)
    for f in files:
        if os.path.isdir(f):
            files.remove(f)
    process(path, os.path.basename(path), files)


def archive(path):
    """compress a directory"""
    process_dir(path)
    subprocess.call(['tar', '-czf', path+'.tar.gz', path])
    shutil.rmtree(path)


#------------------------------------------------------------------------

def main(args):
    
    global commit
    walk = 0    
    paths = []
    
    for arg in args:
        if os.path.isdir(arg):
            paths.append(arg)
        else:
            [key, equal, value] = arg.partition('=')
            if key=='' or equal!='=' or value=='':
                sys.stderr.write("Error: I do not understand `%s'\n" % arg)
            #sys.exit()
            if key=='commit':
                commit = int(value)
            elif key=='walk':
                walk = int(value)
    
    
    if not paths:
        sys.stderr.write("Error: you should specify at least one directory\n")
        sys.exit()
    
    for path in paths:
        try:
            if walk:
                for path, dirs, files in os.walk(path, topdown=False):
                    process(path, dirs, files)
            else:
                archive(path)
        except Exception as e:
            sys.stderr.write("Error in `%s': %s\n" % (path, repr(e)));


#------------------------------------------------------------------------


if __name__ == "__main__":
    if len(sys.argv) < 2 or sys.argv[1].endswith("help"):
        print(__doc__)
    else:
        main(sys.argv[1:])





#!/usr/bin/env python3
#
# FJN, Sainsbury Laboratory, Cambridge University
# 4.6.2019


"""
    Read data from file and save plot 
    
Syntax:
    
    plot.py [filename]
    
"""

import sys, math, os
from random import random

try:
    import matplotlib
    import matplotlib.pyplot as plt
except ImportError:
    print("  Error: could not load matplotlib in python " + sys.version)
    sys.exit()


#set default font size for plots
font = {'family':'arial', 'weight':'normal', 'size': 18}
matplotlib.rc('font', **font)


def random_color():
    """ Return a darkish random RGB triplet"""
    while True:
        R = random()
        G = random()
        B = random()
        if R + G + B < 2:
            break
    return (R, G, B)


def read(filename):
    """
        Read numeric data from file
    """
    data = []
    with open(filename, 'r') as f:
        for line in f:
            if line and line[0]!='%':
                s=(float(x) for x in line.split())
                data.append(s)
    return data


def plot_data(data):
    """ plot data on new graph """
    # repackage data:
    val = list(zip(*data))
    t = val[0]
    for v in val[1:]:
        plt.plot(t, v, '-', linewidth=2, color=random_color())
        #plt.ylim(0, max(v))


def plot(files):
    """ plot data from multiple files and save image """
    fig = plt.figure(figsize=(9, 6))
    ax = plt.axes()
    ax.xaxis.set_major_locator(plt.MaxNLocator(12))
    ax.yaxis.set_major_locator(plt.MaxNLocator(12))
    for f in files:
        data = read(f)
        plot_data(data)
    plt.xlabel('Time')
    plt.ylabel('Molecules')
    plt.title('Simulation')
    fig.tight_layout()
    plt.show()
    plt.savefig('data', dpi=100)
    plt.close()

#-------------------------------------------------------------------------------

def main(args):
    files = []
    # examine command line arguments
    for arg in args:
        if os.path.isfile(arg):
            files.append(arg)
        else:
            sys.stderr.write("Unexpected argument `%s'\n" % arg)
            sys.exit()
    # set default
    if not files:
        files = ['data.txt']
    # process all files
    plot(files)


if __name__ == "__main__":
    if len(sys.argv) > 1 and sys.argv[1]=='help':
        print(__doc__)
    else:
        main(sys.argv[1:])




#!/usr/bin/env python3
#
# make_image.py:
# create the image for one frame in cytosim's result file
#
# copyright F. Nedelec, 2008-2010
#

"""
    Calls executable to generate an image in each specified directory

Syntax:

    make_image.py executable-with-arguments [options] [directories]
 
    The current directory is used if none is specified.
     
Options are specified as 'option=value', without space around the '=' sign.
Existing options and their values:

    format    png, ppm             file format (default = 'png')
    output    file-name            name of file (default = image.png)
    lazy      0 or 1 (default=1)   bail out if output file exists

Examples: 

    make_image.py 'play frame=10' run*
    make_image.py 'play frame=100 window_size=512,256' lazy=0 run*

F. Nedelec, March 19 2011 and before. Revised Sept-Nov 2012.
"""


try:
    import sys, os, subprocess
except ImportError:
    print("  Error: could not load necessary python modules\n")
    sys.exit()


# some parameters:
playexe = []
formats = [ 'png', 'ppm', 'tga' ]
format  = 'png'
output  = 'image.'+format
lazy    = 1

err = sys.stderr

#------------------------------------------------------------------------

def is_image(s):
    return s.startswith('image') and s.endswith(format)


def makeImages(format):
    """call executable to generate one image in the current directory"""
    val = subprocess.call(playexe + ['image', 'image_format='+format], stderr=None)
    if val != 0:
        err.write("`%s' failed with value %i\n" % (playexe[0], val))
        return [];
    files = []
    with os.scandir() as it:
        for e in it:
            if e.is_file() and is_image(e.name):
                files.append(e.name)
    if not files:
        raise IOError("'%s' did not produce any image!" % playexe[0])
    #newest = max(images, key=lambda x: os.stat(x).st_mtime)
    return files


def makeImagesUnzip(format, input_file='objects.cmo'):
    """unzip cytosim's input if necessary to make the images"""
    rmfile=False
    if not os.path.isfile(input_file):
        input_file_gz = input_file + '.gz'
        if os.path.isfile(input_file_gz):
            cmd = "gunzip %s -c > %s" % (input_file_gz, input_file)
            subprocess.call(cmd, shell=True)
            rmfile=True
    if not os.path.isfile(input_file):
        err.write("file '%s' not found!\n" % input_file)
        return []
    images = makeImages(format)
    if rmfile and os.path.isfile(input_file_gz):
        os.remove(input_file)
    return images

#------------------------------------------------------------------------

def process(dirpath, filenames):
    """make image in directory dirpath"""
    if ( output in filenames ) and lazy:
        print("Folder '%s' already has %s" % (dirpath, output))
        return
    try:
        image = makeImagesUnzip(format)
        if len(image) == 0:
            err.write("no image produced in `%s'\n"%dirpath)
        elif len(image) == 1:
            os.rename(image[0], output)
            print(image[0] + ' -> ' + output)
    except Exception as e:
        err.write("Error in `%s': %s\n" % (dirpath, repr(e)))


def process_dir(dirpath):
    """call process() with appropriate arguments"""
    files = []
    with os.scandir() as it:
        for e in it:
            if e.is_file():
                files.append(e.name)
    process(dirpath, files)


#------------------------------------------------------------------------

def main(args):
    """process command line arguments"""
    global playexe, format, output, lazy, codec, colors
    paths = []
    
    playexe = args[0].split()
    arg0 = os.path.expanduser(playexe[0])
    if os.access(arg0, os.X_OK) and os.path.isfile(arg0):
        playexe[0] = os.path.abspath(arg0)
    else:
        err.write("Error: you must specify an executable on the command line\n")
        sys.exit()
    
    for arg in args[1:]:
        [key, equal, value] = arg.partition('=')
        
        if key=='' or equal!='=' or value=='':
            if os.path.isdir(arg):
                paths.append(arg)
            else:
                err.write("ignored '%s' on command line\n" % arg)
        else:
            if key=='output':
                output = value
            elif key=='format':
                format = value
                output = 'image.'+format
                if not format in formats:
                    err.write("Invalid image format '%s'\n" % format)
            elif key=='colors':
                colors = int(value)
            elif key=='lazy':
                lazy = int(value)
            else:
                err.write("ignored '%s' on command line\n" % arg)
    
    if not paths:
        paths.append('.')
    
    cwd = os.getcwd()
    for p in paths:
        print("make_image.py "+"- "*32+p)
        os.chdir(p)
        process_dir(p)
        os.chdir(cwd)


if __name__ == "__main__":
    if len(sys.argv) < 2 or sys.argv[1].endswith("help"):
        print(__doc__)
    else:
        main(sys.argv[1:])





#!/usr/bin/env python3
#
# scan.py executes a given command sequentially or in parallel, in specified directories
#
# Copyright  F. Nedelec and S. Dmitrieff; 2007--2022

"""
    Execute specified command in given directories, sequentially or in parallel,
    using independent threads.
 
Syntax:

    scan.py command [-][+] directory1 [directory2] [directory3] [...] [jobs=INTEGER]
    
    if '-' is specified, output is limited to what the command does
    if '+' is specified, the directory path is printed without decoration
    if 'jobs' is set, run in parallel using specified number of threads

Examples:
    
    scan.py 'play image' run*
    scan.py 'play image' run* jobs=2
    
    
F. Nedelec 02.2011, 09.2012, 03.2013, 01.2014, 06.2017, 07.2021, 21.03.2022, 8.04.2022
S. Dmitreff 06.2017
"""

try:
    import sys, os, subprocess
except ImportError:
    sys.stderr.write("Error: could not load necessary python modules\n")
    sys.exit()

err = sys.stderr
out = sys.stdout
verbose = 2

#------------------------------------------------------------------------

def assemble(path, lines, verb):
    """
    Assembles lines for output according to 'verb' argument
    """
    res = ''
    if verb == 2:
        for s in lines:
            res += s
    elif verb == 1:
        res = os.path.basename(path) + " "
        for s in lines:
            res += s.replace('\n', ' ')
        res += '\n'
    else:
        for s in lines:
            res += s
    return res


def execute(tool, path, verb):
    """
    run executable in specified directory
    """
    lines = []
    try:
        os.chdir(path)
        if verb == 2:
            sys.stderr.write('-  '*24+path+"\n")
        sub = subprocess.Popen(tool, shell=True, stdout=subprocess.PIPE)
        for s in sub.stdout:
            lines.append(s.decode())
        sub.stdout.close()
    except Exception as e:
        err.write("Error: %s\n" % repr(e))
    res = assemble(path, lines, verb)
    out.write(res)
    out.flush()


def worker(queue):
    """
    run executable taking argument from queue
    """
    while True:
        try:
            t, p, v = queue.get(True, 1)
        except:
            break;
        execute(t, p, v)


def main(args):
    """
        read command line arguments and process command
    """
    global verbose
    
    if args[0] == '-':
        verbose = 0
        args.pop(0)
    elif args[0] == '+':
        verbose = 1
        args.pop(0)
    
    try:
        tool = args[0]
    except:
        err.write("Missing command: scan.py command [-][+] directory1 [directory2]...\n")
        return 1

    njobs = 1
    paths = []
    for arg in args[1:]:
        [key, equal, val] = arg.partition('=')
        if os.path.isdir(arg):
            paths.append(os.path.abspath(arg))
        elif key == 'nproc' or key == 'njobs' or key == 'jobs':
            njobs = int(val)
        elif arg == '-':
            verbose = 0
        elif arg == '+':
            verbose = 1
        else:
            err.write("  Warning: unexpected argument `%s'\n" % arg)
            sys.exit()

    if not paths:
        err.write("Missing directories: scan.py command [-][+] directory1 [directory2]...\n")
        err.write(" (scan.py would execute `%s`)\n"%tool)
        return 2
    
    njobs = min(njobs, len(paths))
    
    if njobs > 1:
        #process in parallel with child threads:
        try:
            from multiprocessing import Process, Queue
            queue = Queue()
            for p in paths:
                queue.put((tool, p, verbose))
            jobs = []
            for n in range(njobs):
                j = Process(target=worker, args=(queue,))
                jobs.append(j)
                j.start()
            # wait for completion of all jobs:
            for j in jobs:
                j.join()
                j.close()
            return 0
        except ImportError:
            err.write("Warning: multiprocessing module unavailable\n")
    #process sequentially:
    for p in paths:
        execute(tool, p, verbose)
    return 0

#------------------------------------------------------------------------

if __name__ == "__main__":
    if len(sys.argv) < 3 or sys.argv[1].endswith("help"):
        print(__doc__)
    else:
        main(sys.argv[1:])



#!/usr/bin/env python
#
# battery_test.py
#
# Copyright F. Nedelec, March 19 2011 --- 22.05.2018

"""
battery_test.py:

    run a battery of .cym files to check cytosim

Example - live:

    battery_test.py bin/play live cym/*.cym

Example - runs:

    battery_test.py bin/sim cym/*.cym
    make_image.py 'play frame=100 window_size=512,512' *_cym

F. Nedelec, March-June 2011 - Feb 2013
"""

import shutil, sys, os, subprocess

#------------------------------------------------------------------------

def run_live(file, executable):
    """run live test"""
    print(file.center(100, '~'))
    cmd = executable + ['live', file]
    val = subprocess.call(cmd)
    if val != 0:
        print('returned %i' % val)

#------------------------------------------------------------------------

def run(file, executable):
    """run test in separate directory"""
    cdir = os.getcwd()
    name = os.path.split(file)[1]
    wdir = 'run_'+name.partition('.')[0];
    
    try:
        os.mkdir(wdir)
    except OSError:
        print('skipping  '+file)
        return
    
    shutil.copyfile(file, os.path.join(wdir, 'config.cym'))
    os.chdir(wdir)
    print('- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - '+name)
    
    val = subprocess.call(executable)
    if val != 0:
        print('returned %i' % val)
    
    os.chdir(cdir);

#------------------------------------------------------------------------

def main(args):
    " run cytosim for many files"
    files = []
    live = False;
    err = sys.stderr;

    executable = args[0].split()
    if os.access(executable[0], os.X_OK):
        executable[0] = os.path.abspath(executable[0])
    else:
        err.write("Error: you must specify an executable on the command line\n")
        sys.exit()
    
    for arg in args[1:]:
        if os.path.isfile(arg):
            files.append(os.path.abspath(arg))
        elif arg=='live' or arg=='live=1':
            live = True
        else:
            err.write("Ignored`"+arg+"' on the command line\n")
    
    if not files:
        print("You must specify config files!")
        sys.exit()

    if live:
        for f in files:
            run_live(f, executable)
    else:
        executable.append('-')
        for f in files:
            run(f, executable)


#------------------------------------------------------------------------

if __name__ == "__main__":
    if len(sys.argv)<2 or sys.argv[1].endswith("help"):
        print(__doc__)
    else:
        main(sys.argv[1:])




#!/usr/bin/env python3
#
# collect.py renames files from subdirectories
#
# Copyright F. Nedelec, 2007--2021


"""
Synopsis:
    
    Rename files or folders following a pattern containing an integer index,
    as in 'image0001.png'. The file will be moved in the current directory
    
    The number in the file name is incremented automatically for each file, and
    also if files with this name already exist. Thus pre-existing files are not 
    overwritten, such that 'collect.py' can be used to pool together many similar
    files in a common directory.
    
Syntax:
    
    collect.py PATTERN [INTEGER] [--copy] PATH1 [PATH2] [PATH3] ...

Arguments:
    
    PATTERN specifies the name of the output files, and should contain a variable
    part that will be replaced by an integer. It can be a 'scanf' compatible 
    pattern such as '%i' or '%0Xi', for example 'image%04i.png'.
    A character '%' repeated multiple times, such as `%%%%` or `%%%%%%`, can
    also be used to specify the size of the integer portion of the name.
    
    The pattern can include a '/' that would indicate a directory, and if this
    directory does not exist, collect.py will create it before moving the file.

    if specified, `--copy` will copy the files/directory instead of moving them
    
    if specified, INTEGER is the first index to be used (default=0)

    PATH1, PATH2, etc. is a list of files or directories

Examples:
    
    collect.py image%%%%.png  *.png
       will rename image files to: image0000.png, image0001.png, etc.
    
    collect.py --copy image%%%%.png 1 run*/image.png
       will copy the image files, starting at index 1

    collect.py run%%%%/config.cym config*.cym
       will create directories `run????` and move the `config*.cym` files into them
    
F. Nedelec, 2012--2021. Last modified 6.10.2021
"""


import sys, shutil, os, curses.ascii


#------------------------------------------------------------------------

def cannonical_pattern(arg):
    """check for repeated '%' character, replacing printf syntax: %04i """
    c = arg.count('%')
    for n in range(c,1,-1):
        if arg.find('%'*n) > 0:
            return arg.replace('%'*n, '%0'+str(n)+'i', 1)
    return arg


def validate_pattern(arg):
    # check validity of the pattern
    if os.path.isfile(arg):
        sys.stderr.write("Error: first argument should be the pattern used to build output file name")
        sys.exit(1)
    try:
        res = ( arg % 0 )
    except:
        arg = cannonical_pattern(arg)
    try:
        res = ( arg % 0 )
    except:
        sys.stderr.write("Error: the pattern should accept an integer: eg. '%04i'\n")
        sys.exit(1)
    for c in res:
        if curses.ascii.isspace(c):
            sys.stderr.write("Error: the pattern includes or generates white space character\n")
            sys.exit(1)
    return arg


def main(args):
    """rename files"""
    do_copy = False
    arg = args[0]
    # check if 'copy' specified before pattern
    if arg=='-c' or arg=='--copy' or arg=='copy=1':
        do_copy = True
        args.pop(0)
    # get pattern for moving files:
    pattern = validate_pattern(args.pop(0))
    # parse arguments:
    idx = 0
    paths = []
    for arg in args:
        if arg=='-c' or arg=='--copy' or arg=='copy=1':
            do_copy = True
        elif arg.isdigit():
            idx = int(arg)
        elif os.path.isfile(arg) or os.path.isdir(arg):
            paths.append(arg)
        else:
            sys.stderr.write("Error: '%s' is not a file or directory" % arg)
            return 1
    # process all files:
    res = []
    for src in paths:
        while idx < 1000000:
            dst = pattern % idx
            idx += 1
            if dst == src:
                res.append(dst)
                break
            if not os.path.exists(dst):
                #make directory if name includes a non-existent directory:
                dir = os.path.dirname(dst)
                if dir and not os.path.isdir(dir):
                    os.mkdir(dir)
                # process file:
                if do_copy:
                    shutil.copytree(src, dst)
                else:
                    os.rename(src, dst)
                res.append(dst)
                print("%s -> %s" % (src, dst))
                break
    return res


#------------------------------------------------------------------------

if __name__ == "__main__":
    if len(sys.argv) < 2 or sys.argv[1].endswith("help"):
        print(__doc__)
    else:
        main(sys.argv[1:])





# pyned.py contains often used python subroutines
# it is not executable by itself
# Copyright F. Nedelec, 2017--2019

import os, subprocess, math, copy;


def random_color():
    R=random.random()
    G=random.random()
    B=random.random()
    return (R, G, B)


def sqr(x):
    return x * x;


def uncode(arg):
    try:
        return arg.decode('utf-8')
    except:
        return arg


def szudzik(a, b):
    """
        Matthew Szudzik's pairing function
    """
    if a >= b:
        return a * a + a + b
    else:
        return a + b * b


def kizduzs(z):
    """
        Reverse Matthew Szudzik's pairing function
    """
    b = math.floor(math.sqrt(z))
    a = z - b * b;
    if a < b:
        return ( a, b )
    else:
        return ( b, a - b )



def format_line(data):
    """
        Convert line of data to column-aligned string
    """
    res = ''
    for x in data:
        if isinstance(x, float):
            res += ' %9.4f' % x
        elif isinstance(x, str):
            res += ' %9s' % x
        else:
            res += ' %9s' % repr(x)
    return res


def extract_value(s):
    s = s.rstrip(',;')
    try:
        return 1, int(s)
    except:
        pass
    try:
        return 1, float(s)
    except:
        pass
    return 0, 0


def find_differences(left, right):
    """
        Find differences between two files that are numeric values
        Multiple values are concatenated.
        This can be used to compare config files
    """
    if not os.path.isfile(left):
        return 'file not found: ' + left
    if not os.path.isfile(right):
        return 'file not found: ' + right
    res = []
    sub = subprocess.Popen(['diff', left, right], stdout=subprocess.PIPE)
    for stuff in sub.stdout:
        try:
            line = stuff.decode('utf-8')
        except:
            line = stuff
        if line[0] == '>':
            for s in line.split():
                f, v = extract_value(s)
                if f == 1:
                    res.append(v)
    sub.stdout.close()
    return res


def get_column(file, jj):
    """
        Extract the jj-th word from line ii
        If jj is an array, the corresponding values will be concatenated
    """
    res = ' '
    for line in file:
        if not line or line[0] == '%':
            continue
        s = line.split();
        try:
            res += s[jj].rjust(7) + ' '
        except IndexError:
            pass
    return res


def frange(s, e, n):
    i = ( e - s ) / n
    return [ s+i*x for x in range(n) ]



def simple_linear_fit(arg):
    """
    Fit an linear law, minimizing squared residual with vertical offset
    This returns (0,B) where the best fit is B * x
    """
    data = copy.deepcopy(arg)
    s = 0
    sxx = 0
    sxy = 0
    for x, y in data:
        s += 1
        sxx += x * x
        sxy += x * y
    if s > 0:
        b = sxy / sxx
        return ( 0, b )
    else:
        return ()


def linear_fit(arg):
    """
    Fit an linear law, minimizing squared residual with vertical offset
    This returns (A,B) where the best fit is A + B * x
    """
    data = copy.deepcopy(arg)
    s = 0
    sx = 0
    sy = 0
    sxx = 0
    sxy = 0
    for x, y in data:
        s += 1
        sx += x
        sy += y
        sxx += x * x
        sxy += x * y
    if s > 0:
        n = ( s * sxx - sx * sx )
        a = ( sy * sxx - sx * sxy ) / n
        b = ( s * sxy - sx * sy ) / n
        return ( a, b )
    else:
        return ()


def exponential_fit(arg, ybase = 0):
    """
    Fit an exponential law, using robust (second) method described in
        http://mathworld.wolfram.com/LeastSquaresFittingExponential.html
    This returns (A,B) where the best fit is A * exp(B*x)
    """
    data = copy.deepcopy(arg)
    sy = 0
    sxy = 0
    sxxy = 0
    syly = 0
    sxyly = 0
    for x, y in data:
        if y > ybase:
            yb = y - ybase;
            sy += yb
            sxy += x * yb
            sxxy += x * x * yb
            syly += y * math.log(yb)
            sxyly += x * y * math.log(yb)
    if sy > 0:
        a = ( sxxy*syly - sxy*sxyly ) / ( sy*sxxy - sxy*sxy )
        b = ( sy*sxyly - sxy*syly ) / ( sy*sxxy - sxy*sxy )
        return ( math.exp(a), b )
    else:
        return ()


def exponential_model_fit(arg):
    """
    Fit an exponential curve  A*exp(B*x) + C to the data
    This returns (A,B,C)
    """
    data = copy.deepcopy(arg)
    mxy = min(data)
    best_err = math.inf
    best_fit = ()
    for m in range(0, 100):
        C = mxy-m
        A, B = exponential_fit(data, C)
        data = copy.deepcopy(arg)
        err = 0
        for x, y in data:
            err += abs(y-A*math.exp(B*x)-C)
            print(C, err)
        if err < best_err:
            best_err = err
            best_fit = (A,B,C)
    return best_fit


def power_fit(arg, power):
    """
    Fit a powerlaw with given exponent
    This returns (A) where the best fit is A * x^power
    """
    data = copy.deepcopy(arg)
    sxx = 0
    sxy = 0
    for x, y in data:
        sxx += ( x * x ) ** power
        sxy += y * ( x ** power )
    scale = sxy / sxx
    return scale


def powerlaw_fit(arg):
    """
    Fit a powerlaw, direct method as described in:
        http://mathworld.wolfram.com/LeastSquaresFittingPowerLaw.html
    This returns (A,B) where the best fit is A * x^B
    """
    from math import log, exp
    data = copy.deepcopy(arg)
    s = 0
    sx = 0
    sy = 0
    sxx = 0
    sxy = 0
    syy = 0
    for x, y in data:
        if x > 0 and y > 0:
            s   += 1
            sx  += log(x)
            sy  += log(y)
            sxx += log(x) * log(x)
            sxy += log(x) * log(y)
            syy += log(y) * log(y)
    if s > 0:
        n = s * sxy - sx * sy;
        d = s * sxx - sx * sx;
        power = n / d;
        scale = exp(( sy - power * sx ) / s);
        return ( scale, power )
    return ()



# Test
"""Cytosim: Langevin dynamics of active polymer networks

Cytosim is a simulation tool for cytoskeleton and polymers.
"""
# setup.py stolen from mem3dg : https://github.com/RangamaniLabUCSD/Mem3DG
# and from deepmind's tree : https://github.com/deepmind/tree/
import os
import sys
import subprocess
import re
import platform
import shutil
import sysconfig
import setuptools
import pathlib
#import pybind11
from setuptools.command import build_ext
from setuptools import setup
from setuptools import find_packages
#from pybind11.setup_helpers import Pybind11Extension

version = "0.0.2"
cmake_args=[]


if('CONDA_PREFIX' in os.environ):
    print("Setting library search path (CMAKE_PREFIX_PATH): %s"%(os.environ['CONDA_PREFIX']))
    cmake_args.append('-DCMAKE_PREFIX_PATH=%s'%(os.environ['CONDA_PREFIX']))

DOCLINES = __doc__.split("\n")

CLASSIFIERS = """\
Development Status :: 3 - Alpha
Environment :: Console
Intended Audience :: Science/Research
License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)
Natural Language :: English
Operating System :: OS Independent
Programming Language :: C++
Programming Language :: Python :: 3 :: Only
Programming Language :: Python :: Implementation :: CPython
Topic :: Scientific/Engineering :: Chemistry
Topic :: Scientific/Engineering :: Mathematics
Topic :: Scientific/Engineering :: Physics
Topic :: Scientific/Engineering :: Visualization
"""




#__DIMENSION__ = 2



class CMakeExtension(setuptools.Extension):
  """An extension with no sources.

  We do not want distutils to handle any of the compilation (instead we rely
  on CMake), so we always pass an empty list to the constructor.
  """

  def __init__(self, name, source_dir=''):
    super().__init__(name, sources=[])
    self.source_dir = os.path.abspath(source_dir)


class BuildCMakeExtension(build_ext.build_ext):
  """Our custom build_ext command.

  Uses CMake to build extensions instead of a bare compiler (e.g. gcc, clang).
  """

  def run(self):
    self._check_build_environment()
    for ext in self.extensions:
      self.build_extension(ext)

  def _check_build_environment(self):
    """Check for required build tools: CMake, C++ compiler, and python dev."""
    try:
      subprocess.check_call(['cmake', '--version'])
    except OSError as e:
      ext_names = ', '.join(e.name for e in self.extensions)
      raise RuntimeError(
          f'CMake must be installed to build the following extensions: {ext_names}'
      ) from e
    print('Found CMake')

  def build_extension(self, ext):
    extension_dir = os.path.abspath(
        os.path.dirname(self.get_ext_fullpath(ext.name)))
    build_cfg = 'Debug' if self.debug else 'Release'
    cmake_args = [
        f'-DPython3_ROOT_DIR={sys.prefix}',
        f'-DPython3_EXECUTABLE={sys.executable}',
        f'-DCMAKE_LIBRARY_OUTPUT_DIRECTORY={extension_dir}',
        f'-DCMAKE_BUILD_TYPE={build_cfg}',
        f'-DMAKE_PYCY=ON',
#        f'-DDIMENSION={__DIMENSION__}'  ##### kept for future use ; broken for now
    ]
    if platform.system() != 'Windows':
      cmake_args.extend([
          f'-DPython3_LIBRARY={sysconfig.get_paths()["stdlib"]}',
          f'-DPython3_INCLUDE_DIR={sysconfig.get_paths()["include"]}',
      ])
    if platform.system() == 'Darwin' and os.environ.get('ARCHFLAGS'):
      osx_archs = []
      if '-arch x86_64' in os.environ['ARCHFLAGS']:
        osx_archs.append('x86_64')
      if '-arch arm64' in os.environ['ARCHFLAGS']:
        osx_archs.append('arm64')
      cmake_args.append(f'-DCMAKE_OSX_ARCHITECTURES={";".join(osx_archs)}')
    os.makedirs(self.build_temp, exist_ok=True)
    subprocess.check_call(
        ['cmake', ext.source_dir] + cmake_args, cwd=self.build_temp)
    subprocess.check_call(
        ['cmake', '--build', '.', f'-j{os.cpu_count()}', '--config', build_cfg],
        cwd=self.build_temp)

    # Force output to <extension_dir>/. Amends CMake multigenerator output paths
    # on Windows and avoids Debug/ and Release/ subdirs, which is CMake default.
    ct_dir = os.path.join(extension_dir, 'cytosim')  # pylint:disable=unreachable
    for cfg in ('Release', 'Debug'):
      cfg_dir = os.path.join(extension_dir, cfg)
      if os.path.isdir(cfg_dir):
        for f in os.listdir(cfg_dir):
          shutil.move(os.path.join(cfg_dir, f), ct_dir)
    # Moving the built executables to the package !
    parent = pathlib.Path(extension_dir).parent.absolute()
    for p in os.listdir(parent):
        pp = os.path.join(parent, p)
        if os.path.isdir(pp) and  p.startswith("temp."):
            for f in os.listdir(pp):
                ff = os.path.join(pp, f)
                if os.access(ff, os.X_OK):
                    if f=="sim":
                        pybin = os.path.join(extension_dir, "pycytoplay")
                        if not os.path.isdir(pybin):
                            os.mkdir(pybin)
                        shutil.move(ff, os.path.join(pybin, "wrapped_sim"))
                    if f == "play":
                        pybin = os.path.join(extension_dir, "pycytoplay")
                        if not os.path.isdir(pybin):
                            os.mkdir(pybin)
                        shutil.move(ff, os.path.join(pybin, "wrapped_play"))



if platform.system() != 'Windows':
    setup(
        name="cytoplay",
	version=version,
        packages=['pycytoplay'],
        include_package_data=True,
        package_dir={"": ".", "pycytoplay": "pycytoplay/package"},
        scripts=["pycytoplay/package/sim","pycytoplay/package/play"],
        description=DOCLINES[0],
        long_description=open("README.md", encoding="utf8").read(),
        long_description_content_type="text/markdown",
        platforms=["Linux", "Mac OS-X", "Unix"],
        classifiers=[c for c in CLASSIFIERS.split("\n") if c],
        keywords="simulation actin microtubule polymer",
        cmdclass=dict(build_ext=BuildCMakeExtension),
        ext_modules=[CMakeExtension('cytoplay', source_dir='pycytoplay')],
        zip_safe=False,
    )
else:
    setup(
        name='cytoplay',
        version=version,
        author="Serge Dmitrieff",
        description="A dummy package for windows",
        packages=['cytoplay'],
        package_dir={'cytoplay': 'dummy'},
    )



#!/usr/bin/env python3
#
# compare_config.py
#
# compare two by two all the provided files and print the one with fewer differences than the cutoff
#
# Copyright F. Nedelec, 2011 - 2016

"""
    Compare two by two all the provided files and print the one with fewer differences

Syntax:

    compare_config.py THRESHOLD files
    
Example:

    compare_config 2 run????/config.cym

Description:

    Compare all config files two by two, and print the pair if they differ
    at most on THRESHOLD lines
    
F. Nedelec, Mar. 2016
"""

from __future__ import print_function

import sys, os, subprocess

threshold = 2

#------------------------------------------------------------------------

def uncode(arg):
    try:
        if isinstance(arg, unicode):
            return str(arg.decode('utf-8'))
    except:
        pass
    return arg


def find_differences(left, right):
    """
        Find differences between the file that are numeric values
        Multiple values are concatenated.
    """
    L = []
    R = []
    sub = subprocess.Popen(['diff', left, right], stdout=subprocess.PIPE)
    while sub.stdout:
        line = uncode(sub.stdout.readline())
        if len(line) < 2:
            break
        if line[0] == '<':
            L.append(line[0:len(line)-1])
        elif line[0] == '>':
            R.append(line[0:len(line)-1])
    sub.stdout.close()
    return L, R


def process(files):
    """
    Compare all file pairs
    """
    for f in files:
        for g in files:
            if g <= f:
                continue
            L, R = find_differences(f, g)
            if len(L)+len(R) <= threshold:
                fs = f.rstrip('.tpl').rstrip('/config.cym')
                gs = g.rstrip('.tpl').rstrip('/config.cym')
                print(fs.ljust(10) + ' ' + gs.ljust(10), end='')
                if len(L)+len(R) == 0:
                    print(': no differences')
                elif len(L) == 1 and len(R) == 1:
                    L = L[0][2:].strip()
                    R = R[0][2:].strip()
                    print(':   %-40s <>    %-40s' %(L, R))
                else:
                    print('')
                    for d in L:
                        print('                  '+d)
                    for d in R:
                        print('                  '+d)
 

#------------------------------------------------------------------------

def main(args):
    global threshold
    files = []
    
    for arg in args:
        if arg.isdigit():
            threshold = int(arg)
        elif os.path.isdir(arg):
            files.append(arg+'/config.cym')
        elif os.path.isfile(arg):
            files.append(arg)
        else:
            sys.stderr.write("  Error: unexpected argument `%s'\n" % arg)
            sys.exit()
    
    if len(files) < 2:
        sys.stderr.write("  Error: two files at least should be specified\n")
        sys.exit()
    
    process(files)


if __name__ == "__main__":
    if len(sys.argv) > 1 and sys.argv[1].endswith("help"):
        print(__doc__)
    else:
        main(sys.argv[1:])





#!/usr/bin/env python3
#
# Make a plot using matplotlib
#
# F. Nedelec, 13-15 November 2013
#
# How I installed matplotlib in Jan 2015 (Mac osx 10.10):
#
# brew install python3 --framework
# pip3 install numpy
# pip3 install nose
# pip3 install matplotlib
#
# Alternative from source:
# git clone git://github.com/matplotlib/matplotlib.git
# cd matplotlib
# python3 setup.py build
# sudo python3 setup.py install
# python3 setup.py clean
#


"""
    Make plots
    
Syntax:
    
    make_plots.py DIRECTORY-PATH
    
Description:
    
"""

import sys, os, math, subprocess
import numpy
import matplotlib.pyplot as plt
import read_config
#matplotlib.use('SVG')

fts = 14
out = sys.stderr

def load_data(file):
    try:
        return numpy.loadtxt(file, comments='%', unpack=True)
    except:
        return []

def image(x, p):
	while x > p:
		x = x - 2*p
	while x < -p:
		x = x + 2*p
	return x


def plot_diameter(cdata, rdata):
    plt.figure(figsize=(3.84, 2.56), dpi=100)
    plt.plot(cdata[0], cdata[1], 'ko')
    plt.plot(rdata[0], rdata[1], 'b-', linewidth=4)
    plt.xlabel('Time (s)', fontsize=fts)
    plt.ylabel(r'Diameter ($\mu m$)', fontsize=fts)
    #plt.title('Closure', fontsize=fts)
    plt.tight_layout()
    #plt.xlim([-5, 5])
    #plt.grid(True)


def plot_positions(xdata, xlim, xinfo):
    plt.figure(figsize=(3.84, 2.56), dpi=100)
    n, bins, patches = plt.hist(xdata, 20, normed=1)
    ax = plt.gca()
    ax.yaxis.set_visible(False)
    plt.xlabel('Position (%s)' % xinfo, fontsize=fts)
    plt.ylabel('Amount', fontsize=fts)
    plt.title('Histogram', fontsize=fts)
    plt.tight_layout()
    plt.xlim(xlim)
    #plt.grid(True)


def parse(dirpath):
    """
    Work in current directory
    """
    pile = read_config.parse('config.cym')
    shape = read_config.value(pile, ['set', 'space', 'cell', 'geometry'])

    if shape.startswith('wall'):
        subprocess.call(['reportW', 'space'], stdout=open('radius.txt', 'w'), stderr=None)
        rdata = load_data('radius.txt')
        if len(rdata) > 0:
            cdata = get_data('/Users/nedelec/tmp/closure_diameter.txt')
            data = [ rdata[2], [ 2*x for x in rdata[3] ] ]
            plot_diameter(cdata, data)
            plt.savefig('diameter.png')
            plt.close()
        #
        subprocess.call(['reportW', 'bead:all', 'frame=1999'], stdout=open('beads.txt', 'w'), stderr=None)
        data = load_data('beads.txt')
        data = [ math.atan2(data[3][n], data[2][n]) for n in range(len(data[0])) ]
        if len(data) > 0:
            plot_positions(data, [-3.1416, 3.1416], 'radian')
            plt.savefig('position.png')
            plt.close()

    if shape.startswith('strip'):
        subprocess.call(['reportW', 'bead:all', 'frame=999'], stdout=open('beads.txt', 'w'), stderr=None)
        data = load_data('beads.txt')
        if len(data) > 0:
            data = [ image(x, 5) for x in data[2] ]
            plot_positions(data, [-5, 5], 'um')
            plt.savefig('position.png')
            plt.close()


#------------------------------------------------------------------------

def main(args):
    paths = []
    
    for arg in args:
        if os.path.isdir(arg):
            paths.append(arg)
        else:
            out.write("  Error: unexpected argument `%s'\n" % arg)
            sys.exit()
    
    if not paths:
        parse('.')
    else:
        cdir = os.getcwd()
        for p in paths:
            os.chdir(p)
            sys.stdout.write('- '*32+p"\n")
            try:
                parse(p)
            except Exception as e:
                out.write("Error: %s\n" % repr(e))
            os.chdir(cdir)

#------------------------------------------------------------------------

if __name__ == "__main__":
    if len(sys.argv) < 2 or sys.argv[1].endswith("help"):
        print(__doc__)
    else:
        main(sys.argv[1:])

